@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 u"""Configuration file parser.
@@ -124,10 +123,7 @@ ConfigParser -- responsible for parsing a list of
 from __future__ import with_statement
 from collections import MutableMapping
 from io import open
-try:
-    from collections import OrderedDict as _default_dict
-except ImportError:
-    from ordereddict import OrderedDict as _default_dict
+from collections import OrderedDict as _default_dict
 import functools
 import io
 import itertools
@@ -166,10 +162,23 @@ class Error(Exception):
         """
         self.__message = value
 
+    def _get_args(self):
+        u"""Getter for 'message'; needed only to override deprecation in
+        BaseException.
+        """
+        return self.__args
+
+    def _set_args(self, value):
+        u"""Setter for 'message'; needed only to override deprecation in
+        BaseException.
+        """
+        self.__args = value
+
     # BaseException.message has been deprecated since Python 2.6.  To prevent
     # DeprecationWarning from popping up over this pre-existing attribute, use
     # a new property that takes lookup precedence.
     message = property(_get_message, _set_message)
+    args = property(_get_args, _set_args)
 
     def __init__(self, msg=u''):
         self.message = msg
@@ -187,7 +196,7 @@ class NoSectionError(Error):
     def __init__(self, section):
         Error.__init__(self, u'No section: %r' % (section,))
         self.section = section
-        self.args = (section, )
+        self.args = (section,)
 
 
 class DuplicateSectionError(Error):
@@ -316,27 +325,7 @@ class ParsingError(Error):
         Error.__init__(self, u'Source contains parsing errors: %s' % source)
         self.source = source
         self.errors = []
-        self.args = (source, )
-
-    @property
-    def filename(self):
-        u"""Deprecated, use `source'."""
-        warnings.warn(
-            u"The 'filename' attribute will be removed in future versions.  "
-            u"Use 'source' instead.",
-            DeprecationWarning, stacklevel=2
-        )
-        return self.source
-
-    @filename.setter
-    def filename(self, value):
-        u"""Deprecated, user `source'."""
-        warnings.warn(
-            u"The 'filename' attribute will be removed in future versions.  "
-            u"Use 'source' instead.",
-            DeprecationWarning, stacklevel=2
-        )
-        self.source = value
+        self.args = (source,)
 
     def append(self, lineno, line):
         self.errors.append((lineno, line))
@@ -356,7 +345,6 @@ class MissingSectionHeaderError(ParsingError):
         self.line = line
         self.args = (filename, lineno, line)
 
-
 # Used in parser getters to indicate the default behaviour when a specific
 # option is not found it to raise an exception. Created to enable `None' as
 # a valid fallback value.
@@ -402,8 +390,8 @@ class BasicInterpolation(Interpolation):
         return u''.join(L)
 
     def before_set(self, parser, section, option, value):
-        tmp_value = value.replace(u'%%', u'') # escaped percent signs
-        tmp_value = self._KEYCRE.sub(u'', tmp_value) # valid syntax
+        tmp_value = value.replace(u'%%', u'')  # escaped percent signs
+        tmp_value = self._KEYCRE.sub(u'', tmp_value)  # valid syntax
         if u'%' in tmp_value:
             raise ValueError(u"invalid interpolation syntax in %r at "
                              u"position %d" % (value, tmp_value.find(u'%')))
@@ -429,7 +417,8 @@ class BasicInterpolation(Interpolation):
             elif c == u"(":
                 m = self._KEYCRE.match(rest)
                 if m is None:
-                    raise InterpolationSyntaxError(option, section,
+                    raise InterpolationSyntaxError(
+                        option, section,
                         u"bad interpolation variable reference %r" % rest)
                 var = parser.optionxform(m.group(1))
                 rest = rest[m.end():]
@@ -462,8 +451,8 @@ class ExtendedInterpolation(Interpolation):
         return u''.join(L)
 
     def before_set(self, parser, section, option, value):
-        tmp_value = value.replace(u'$$', u'') # escaped dollar signs
-        tmp_value = self._KEYCRE.sub(u'', tmp_value) # valid syntax
+        tmp_value = value.replace(u'$$', u'')  # escaped dollar signs
+        tmp_value = self._KEYCRE.sub(u'', tmp_value)  # valid syntax
         if u'$' in tmp_value:
             raise ValueError(u"invalid interpolation syntax in %r at "
                              u"position %d" % (value, tmp_value.find(u'%')))
@@ -489,12 +478,12 @@ class ExtendedInterpolation(Interpolation):
             elif c == u"{":
                 m = self._KEYCRE.match(rest)
                 if m is None:
-                    raise InterpolationSyntaxError(option, section,
+                    raise InterpolationSyntaxError(
+                        option, section,
                         u"bad interpolation variable reference %r" % rest)
                 path = m.group(1).split(u':')
                 rest = rest[m.end():]
                 sect = section
-                opt = option
                 try:
                     if len(path) == 1:
                         opt = parser.optionxform(path[0])
@@ -532,7 +521,7 @@ class LegacyInterpolation(Interpolation):
     def before_get(self, parser, section, option, value, vars):
         rawval = value
         depth = MAX_INTERPOLATION_DEPTH
-        while depth:                    # Loop through this until it's done
+        while depth:  # Loop through this until it's done
             depth -= 1
             if value and u"%(" in value:
                 replace = functools.partial(self._interpolation_replace,
@@ -605,20 +594,41 @@ class RawConfigParser(MutableMapping):
     def __init__(self, defaults=None, dict_type=_default_dict,
                  allow_no_value=False, **_3to2kwargs):
 
-        if 'interpolation' in _3to2kwargs: interpolation = _3to2kwargs['interpolation']; del _3to2kwargs['interpolation']
-        else: interpolation = _UNSET
-        if 'default_section' in _3to2kwargs: default_section = _3to2kwargs['default_section']; del _3to2kwargs['default_section']
-        else: default_section = DEFAULTSECT
-        if 'empty_lines_in_values' in _3to2kwargs: empty_lines_in_values = _3to2kwargs['empty_lines_in_values']; del _3to2kwargs['empty_lines_in_values']
-        else: empty_lines_in_values = True
-        if 'strict' in _3to2kwargs: strict = _3to2kwargs['strict']; del _3to2kwargs['strict']
-        else: strict = False
-        if 'inline_comment_prefixes' in _3to2kwargs: inline_comment_prefixes = _3to2kwargs['inline_comment_prefixes']; del _3to2kwargs['inline_comment_prefixes']
-        else: inline_comment_prefixes = None
-        if 'comment_prefixes' in _3to2kwargs: comment_prefixes = _3to2kwargs['comment_prefixes']; del _3to2kwargs['comment_prefixes']
-        else: comment_prefixes = (u'#', u';')
-        if 'delimiters' in _3to2kwargs: delimiters = _3to2kwargs['delimiters']; del _3to2kwargs['delimiters']
-        else: delimiters = (u'=', u':')
+        if 'interpolation' in _3to2kwargs:
+            interpolation = _3to2kwargs['interpolation']
+            del _3to2kwargs['interpolation']
+        else:
+            interpolation = _UNSET
+        if 'default_section' in _3to2kwargs:
+            default_section = _3to2kwargs['default_section']
+            del _3to2kwargs['default_section']
+        else:
+            default_section = DEFAULTSECT
+        if 'empty_lines_in_values' in _3to2kwargs:
+            empty_lines_in_values = _3to2kwargs['empty_lines_in_values']
+            del _3to2kwargs['empty_lines_in_values']
+        else:
+            empty_lines_in_values = True
+        if 'strict' in _3to2kwargs:
+            strict = _3to2kwargs['strict']
+            del _3to2kwargs['strict']
+        else:
+            strict = False
+        if 'inline_comment_prefixes' in _3to2kwargs:
+            inline_comment_prefixes = _3to2kwargs['inline_comment_prefixes']
+            del _3to2kwargs['inline_comment_prefixes']
+        else:
+            inline_comment_prefixes = None
+        if 'comment_prefixes' in _3to2kwargs:
+            comment_prefixes = _3to2kwargs['comment_prefixes']
+            del _3to2kwargs['comment_prefixes']
+        else:
+            comment_prefixes = (u'#', u';')
+        if 'delimiters' in _3to2kwargs:
+            delimiters = _3to2kwargs['delimiters']
+            del _3to2kwargs['delimiters']
+        else:
+            delimiters = (u'=', u':')
         self._dict = dict_type
         self._sections = self._dict()
         self._defaults = self._dict()
@@ -643,7 +653,7 @@ class RawConfigParser(MutableMapping):
         self._strict = strict
         self._allow_no_value = allow_no_value
         self._empty_lines_in_values = empty_lines_in_values
-        self.default_section=default_section
+        self.default_section = default_section
         self._interpolation = interpolation
         if self._interpolation is _UNSET:
             self._interpolation = self._DEFAULT_INTERPOLATION
@@ -772,13 +782,23 @@ class RawConfigParser(MutableMapping):
         )
         self.read_file(fp, source=filename)
 
-    def get(self, section, option, **_3to2kwargs):
-        if 'fallback' in _3to2kwargs: fallback = _3to2kwargs['fallback']; del _3to2kwargs['fallback']
-        else: fallback = _UNSET
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+    def get(self, *args, **_3to2kwargs):
+        section, option = args
+        if 'fallback' in _3to2kwargs:
+            fallback = _3to2kwargs['fallback']
+            del _3to2kwargs['fallback']
+        else:
+            fallback = _UNSET
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         u"""Get an option value for a given section.
 
         If `vars' is provided, it must be a dictionary. The option is looked up
@@ -819,12 +839,21 @@ class RawConfigParser(MutableMapping):
         return conv(self.get(section, option, **kwargs))
 
     def getint(self, section, option, **_3to2kwargs):
-        if 'fallback' in _3to2kwargs: fallback = _3to2kwargs['fallback']; del _3to2kwargs['fallback']
-        else: fallback = _UNSET
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'fallback' in _3to2kwargs:
+            fallback = _3to2kwargs['fallback']
+            del _3to2kwargs['fallback']
+        else:
+            fallback = _UNSET
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         try:
             return self._get(section, int, option, raw=raw, vars=vars)
         except (NoSectionError, NoOptionError):
@@ -834,12 +863,21 @@ class RawConfigParser(MutableMapping):
                 return fallback
 
     def getfloat(self, section, option, **_3to2kwargs):
-        if 'fallback' in _3to2kwargs: fallback = _3to2kwargs['fallback']; del _3to2kwargs['fallback']
-        else: fallback = _UNSET
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'fallback' in _3to2kwargs:
+            fallback = _3to2kwargs['fallback']
+            del _3to2kwargs['fallback']
+        else:
+            fallback = _UNSET
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         try:
             return self._get(section, float, option, raw=raw, vars=vars)
         except (NoSectionError, NoOptionError):
@@ -849,12 +887,21 @@ class RawConfigParser(MutableMapping):
                 return fallback
 
     def getboolean(self, section, option, **_3to2kwargs):
-        if 'fallback' in _3to2kwargs: fallback = _3to2kwargs['fallback']; del _3to2kwargs['fallback']
-        else: fallback = _UNSET
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'fallback' in _3to2kwargs:
+            fallback = _3to2kwargs['fallback']
+            del _3to2kwargs['fallback']
+        else:
+            fallback = _UNSET
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         try:
             return self._get(section, self._convert_to_boolean, option,
                              raw=raw, vars=vars)
@@ -887,8 +934,8 @@ class RawConfigParser(MutableMapping):
         if vars:
             for key, value in vars.items():
                 d[self.optionxform(key)] = value
-        value_getter = lambda option: self._interpolation.before_get(self,
-            section, option, d[option], d)
+        value_getter = lambda option: self._interpolation.before_get(
+            self, section, option, d[option], d)
         if raw:
             value_getter = lambda option: d[option]
         return [(option, value_getter(option)) for option in d.keys()]
@@ -949,7 +996,7 @@ class RawConfigParser(MutableMapping):
             d = self._delimiters[0]
         if self._defaults:
             self._write_section(fp, self.default_section,
-                                    self._defaults.items(), d)
+                                self._defaults.items(), d)
         for section in self._sections:
             self._write_section(fp, section,
                                 self._sections[section].items(), d)
@@ -1018,7 +1065,7 @@ class RawConfigParser(MutableMapping):
         return key == self.default_section or self.has_section(key)
 
     def __len__(self):
-        return len(self._sections) + 1 # the default section
+        return len(self._sections) + 1  # the default section
 
     def __iter__(self):
         # XXX does it break when underlying container state changed?
@@ -1042,24 +1089,24 @@ class RawConfigParser(MutableMapping):
         section names.
         """
         elements_added = set()
-        cursect = None                        # None, or a dictionary
+        cursect = None  # None, or a dictionary
         sectname = None
         optname = None
-        lineno = 0
         indent_level = 0
-        e = None                              # None, or an exception
+        e = None  # None, or an exception
         for lineno, line in enumerate(fp, start=1):
             comment_start = sys.maxsize
             # strip inline comments
-            inline_prefixes = dict((p, -1) for p in self._inline_comment_prefixes)
+            inline_prefixes = dict(
+                (p, -1) for p in self._inline_comment_prefixes)
             while comment_start == sys.maxsize and inline_prefixes:
                 next_prefixes = {}
                 for prefix, index in inline_prefixes.items():
-                    index = line.find(prefix, index+1)
+                    index = line.find(prefix, index + 1)
                     if index == -1:
                         continue
                     next_prefixes[prefix] = index
-                    if index == 0 or (index > 0 and line[index-1].isspace()):
+                    if index == 0 or (index > 0 and line[index - 1].isspace()):
                         comment_start = min(comment_start, index)
                 inline_prefixes = next_prefixes
             # strip full line comments
@@ -1075,10 +1122,9 @@ class RawConfigParser(MutableMapping):
                     # add empty line to the value, but only if there was no
                     # comment on the line
                     if (comment_start is None and
-                        cursect is not None and
-                        optname and
-                        cursect[optname] is not None):
-                        cursect[optname].append(u'') # newlines added at join
+                                cursect is not None and
+                            optname and cursect[optname] is not None):
+                        cursect[optname].append(u'')  # newlines added at join
                 else:
                     # empty line marks end of value
                     indent_level = sys.maxsize
@@ -1087,7 +1133,7 @@ class RawConfigParser(MutableMapping):
             first_nonspace = self.NONSPACECRE.search(line)
             cur_indent_level = first_nonspace.start() if first_nonspace else 0
             if (cursect is not None and optname and
-                cur_indent_level > indent_level):
+                        cur_indent_level > indent_level):
                 cursect[optname].append(value)
             # a section header or option header?
             else:
@@ -1118,12 +1164,13 @@ class RawConfigParser(MutableMapping):
                 else:
                     mo = self._optcre.match(value)
                     if mo:
-                        optname, vi, optval = mo.group(u'option', u'vi', u'value')
+                        optname, vi, optval = mo.group(u'option', u'vi',
+                                                       u'value')
                         if not optname:
                             e = self._handle_error(e, fpname, lineno, line)
                         optname = self.optionxform(optname.rstrip())
                         if (self._strict and
-                            (sectname, optname) in elements_added):
+                                    (sectname, optname) in elements_added):
                             raise DuplicateOptionError(sectname, optname,
                                                        fpname, lineno)
                         elements_added.add((sectname, optname))
@@ -1192,12 +1239,21 @@ class RawConfigParser(MutableMapping):
         return self.BOOLEAN_STATES[value.lower()]
 
     def _validate_value_types(self, **_3to2kwargs):
-        if 'value' in _3to2kwargs: value = _3to2kwargs['value']; del _3to2kwargs['value']
-        else: value = u""
-        if 'option' in _3to2kwargs: option = _3to2kwargs['option']; del _3to2kwargs['option']
-        else: option = u""
-        if 'section' in _3to2kwargs: section = _3to2kwargs['section']; del _3to2kwargs['section']
-        else: section = u""
+        if 'value' in _3to2kwargs:
+            value = _3to2kwargs['value']
+            del _3to2kwargs['value']
+        else:
+            value = u""
+        if 'option' in _3to2kwargs:
+            option = _3to2kwargs['option']
+            del _3to2kwargs['option']
+        else:
+            option = u""
+        if 'section' in _3to2kwargs:
+            section = _3to2kwargs['section']
+            del _3to2kwargs['section']
+        else:
+            section = u""
         u"""Raises a TypeError for non-string values.
 
         The only legal non-string value if we allow valueless
@@ -1218,7 +1274,7 @@ class RawConfigParser(MutableMapping):
             if not isinstance(value, basestring):
                 raise TypeError(u"option values must be strings")
         return (unicode(section), unicode(option), (value if value is None
-            else unicode(value)))
+                                                    else unicode(value)))
 
 
 class ConfigParser(RawConfigParser):
@@ -1229,7 +1285,8 @@ class ConfigParser(RawConfigParser):
     def set(self, section, option, value=None):
         u"""Set an option.  Extends RawConfigParser.set by validating type and
         interpolation syntax on the value."""
-        _, option, value = self._validate_value_types(option=option, value=value)
+        _, option, value = self._validate_value_types(option=option,
+                                                      value=value)
         super(ConfigParser, self).set(section, option, value)
 
     def add_section(self, section):
@@ -1270,12 +1327,13 @@ class SectionProxy(MutableMapping):
         return self._parser.get(self._name, key)
 
     def __setitem__(self, key, value):
-        _, key, value = self._parser._validate_value_types(option=key, value=value)
+        _, key, value = self._parser._validate_value_types(option=key,
+                                                           value=value)
         return self._parser.set(self._name, key, value)
 
     def __delitem__(self, key):
         if not (self._parser.has_option(self._name, key) and
-                self._parser.remove_option(self._name, key)):
+                    self._parser.remove_option(self._name, key)):
             raise KeyError(key)
 
     def __contains__(self, key):
@@ -1294,34 +1352,58 @@ class SectionProxy(MutableMapping):
             return self._parser.defaults()
 
     def get(self, option, fallback=None, **_3to2kwargs):
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         return self._parser.get(self._name, option, raw=raw, vars=vars,
                                 fallback=fallback)
 
     def getint(self, option, fallback=None, **_3to2kwargs):
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         return self._parser.getint(self._name, option, raw=raw, vars=vars,
                                    fallback=fallback)
 
     def getfloat(self, option, fallback=None, **_3to2kwargs):
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         return self._parser.getfloat(self._name, option, raw=raw, vars=vars,
                                      fallback=fallback)
 
     def getboolean(self, option, fallback=None, **_3to2kwargs):
-        if 'vars' in _3to2kwargs: vars = _3to2kwargs['vars']; del _3to2kwargs['vars']
-        else: vars = None
-        if 'raw' in _3to2kwargs: raw = _3to2kwargs['raw']; del _3to2kwargs['raw']
-        else: raw = False
+        if 'vars' in _3to2kwargs:
+            vars = _3to2kwargs['vars']
+            del _3to2kwargs['vars']
+        else:
+            vars = None
+        if 'raw' in _3to2kwargs:
+            raw = _3to2kwargs['raw']
+            del _3to2kwargs['raw']
+        else:
+            raw = False
         return self._parser.getboolean(self._name, option, raw=raw, vars=vars,
                                        fallback=fallback)
 
